#
#  gitrelease/builder.rb  --  Build releases
#

require "supplement"
require "supplement/dir"

require "gitrelease"


module GitRelease

  class Builder < Config

    config_re :versionscheme
    config :destdir

    class <<self

      def each_ref since = nil
        since = Time.parse since if String === since
        f = %w(objecttype refname creatordate *creatordate).map { |x|
          x += ":iso8601" if x.ends_with? "date"
          "%(#{x})"
        }.join ","
        `git for-each-ref --perl '--format=[#{f}]'`.each_line { |l|
          y, t, c, d = eval l
          case y
            when "tag"    then c = d
            when "commit" then nil
          end
          next if since and (Time.parse c) < since
          i = new t
          yield i
        }
      end

    end

    def initialize ref, mklatest = nil
      ref = Reference.new ref unless Reference === ref
      @ref = ref
      if @ref.type == "tags" and @ref.short =~ versionscheme then
        @version = $1||$'
      end
      @mklatest = mklatest
    end

    def build
      @version or return
      dir = File.expand_path destdir
      File.directory? dir or Dir.mkdir! dir
      tmpdir do
        each_archive do |src,template|
          name = template.sub "#", @version
          dst = File.expand_path name, dir
          if File.exists? dst then
            raise "Cannot overwrite existing '#{dst}'."
          end
          move_file src, dst
          puts "Created file #{dst}."
          if @mklatest then
            link = template.sub "#", "latest"
            dst = File.expand_path link, dir
            File.symlink? dst and File.unlink dst
            File.symlink name, dst
            puts "Created symlink #{dst}."
          end
        end
      end
    end

    private

    def move_file src, dst
      File.rename src, dst
    rescue Errno::EXDEV
      File.open src do |s|
        b = s.stat.blksize
        File.open dst, "w" do |d|
          while (p = s.read b) do d.write p end
        end
      end
      File.unlink src
    end

    TMPDIR = "/var/tmp"  # :nodoc:

    def tmpdir
      begin
        n = File.join TMPDIR, "gitrelease.%06x" % (rand 0x1000000)
        Dir.mkdir n
      rescue Errno::EEXIST
        retry
      end
      unpack n
      Dir.chdir n do
        yield
      end
    ensure
      Dir.nuke! n
    end

  end

  class Zips < Builder

    CONFIG_SECTION = "zip"

    config :filename

    def initialize *args
      super
    end

    private

    ZIPNAME = "gitrelease-#$$.zip"

    def unpack dir
      @filename = filename
      `git archive --format=zip #@ref >#{dir}/#{ZIPNAME}`
    end

    def each_archive
      yield ZIPNAME, @filename
    end

  end

  class Gems < Builder

    CONFIG_SECTION = "gem"

    private

    def unpack dir
      `git archive --format=tar #@ref | tar -xf - -C #{dir}`
    end

    def each_archive
      require "rubygems/package"
      a = Dir.current.entries.select { |e| e.ends_with? ".gemspec" }
      a.each do |spec|
        b = {}
        `gem build #{spec}`.each_line { |l|
          l =~ /^\s*(\w+):\s*(.*?)\s*$/ and b[ $1.downcase.intern] = $2
        }
        b[ :version] == @version or
          raise "#@ref has a wrong version number: '#{b[ :file]}."
        yield b[ :file], "#{b[ :name]}-#.gem"
      end
    end

  end

end

