#
#  gitrelease/fetch.rb  --  Fetch the privided Packets
#

require "supplement"
require "supplement/dir"
require "rbfind"


module GitRelease

  # == Examples
  #
  # This just fetches from a directory containing tarballs or zip files.
  # Suppose you had the files:
  #
  #   ~devel/tarballs/projecta-1.0.tar.gz
  #   ~devel/tarballs/projecta-1.1.tar.gz
  #   ~devel/tarballs/projecta-latest.tar.gz  ->  projecta-1.1.tar.gz
  #   ~devel/tarballs/projectb-2.0.tar.gz
  #   ~devel/tarballs/projectb-latest.tar.gz  ->  projectb-2.0.tar.gz
  #
  # Then you would fetch them into a directory "pub/myprojects" by:
  #
  #   require "gitrelease/fetch"
  #   include GitRelease
  #
  #   SSH = "devel@repository.local"
  #
  #   p = Fetch.new SSH, "pub"
  #   p.fetch_directory "tarballs", "myprojects"
  #
  # Result:
  #
  #   pub/myprojects/projecta-1.0.tar.gz
  #   pub/myprojects/projecta-1.1.tar.gz
  #   pub/myprojects/projectb-2.0.tar.gz
  #
  #
  # Now, make a gem server and a series of RDoc directories. Suppose your
  # origin were
  #
  #   ~devel/locallybuiltgems/rubybtoola-1.0.gem
  #   ~devel/locallybuiltgems/rubybtoola-1.1.gem
  #   ~devel/locallybuiltgems/rubybtoola-latest.gem  ->  rubybtoola-1.1.gem
  #   ~devel/locallybuiltgems/rubybtoolb-2.0.gem
  #   ~devel/locallybuiltgems/rubybtoolb-latest.gem  ->  rubybtoolb-2.0.gem
  #
  # Say:
  #
  #   p = Fetch.new SSH, "gemserver"
  #   n = Homepages.new "projectsites"
  #   p.fetch_directory "locallybuiltgems", "gems" do |k,v|
  #     n.make_rdoc k, v
  #   end
  #   p.make_gemserver
  #
  # This will produce the following directories to be published by an HTTP
  # server:
  #
  #   gemserver/                Provides all gems by "gem generate_index"
  #   projectsites/rubybtoola   RDoc of rubybtoola-1.1
  #   projectsites/rubybtoolb   RDoc of rubybtoola-2.0
  #
  class Fetch

    class <<self

      def open *args
        i = new *args
        yield i
      end

      def gem_version_2
        if @gem_version_2.nil? then
          v = `gem --version`
          v.chomp!
          @gem_version_2 = v >= "2"
        end
        @gem_version_2
      end

    end

    def initialize ssh, dst
      @ssh, @dst = ssh, dst
    end

    attr_reader :dst

    def fetch_directory remote, local = nil
      local = mk_local local do remote end
      system <<-EOT
        ssh '#@ssh' 'tar -cf - -C #{remote} .' | tar -xf - -C #{local}
      EOT
      latest = {}
      RbFind.open local do |f|
        next unless f.symlink?
        if f.name =~ /-latest\.gem\z/ then
          d = File.dirname f.path
          latest[ $`] = File.join d, f.readlink
        end
        File.unlink f.path
      end
      if block_given? then
        latest.each { |k,v| yield k, v }
        latest.length.nonzero?
      else
        latest
      end
    end

    def make_gemserver
      g = File.join @dst, "gems"
      File.directory? g or raise "#@dst is not a gems directory."
      system "gem generate_index --directory #@dst"
      $?.success? or raise "Indexgenerierung gescheitert mit #{$?.exitstatus}."
    end

    def fetch_file remote, local = nil
      local = mk_local local do "." end
      dir, file = File.split remote
      system <<-EOT
        ssh '#@ssh' 'tar -cf - -C #{dir} #{file}' | tar -xf - -C #{local}
      EOT
    end

    def fetch_gitfiles repo, ref = nil, path = nil, local = nil
      local = mk_local local do
        l = File.basename repo
        l.slice! /\.git\z/
        l
      end
      ref ||= "HEAD"
      system <<-EOT
        git archive --remote=ssh://#@ssh#{repo} --format=tar #{ref} #{path} | \
          tar -xf - -C #{local}
      EOT
    end

    def clean
      File.directory? @dst and Dir.nuke! @dst
    end

    private

    def mk_local local
      local ||= yield
      local = File.join @dst, local
      File.directory? local or Dir.mkdir! local
    end

  end


  # See class +Fetch+ for an example.
  #
  class Homepages

    class <<self
      def open *args
        i = new *args
        yield i
      end
    end

    attr_reader :dst

    def initialize dst
      @dst = dst
    end

    def make_rdoc name, filename
      Dir.mkdir! @dst rescue nil
      tmpdir do
        install_gem filename
        b = File.basename filename
        b.slice! /\.gem\z/
        dst = "#@dst/#{name}"
        File.directory? dst and Dir.nuke! dst
        File.rename "#{TMPDIR}/doc/#{b}/rdoc", dst
      end
    end

    def clean
      File.directory? @dst and Dir.nuke! @dst
    end

    private

    TMPDIR = "geminstall"  # :nodoc:

    def tmpdir
      Dir.mkdir! TMPDIR rescue nil
      yield
    ensure
      Dir.nuke! TMPDIR
    end

    def install_gem filename
      ENV[ "rake"] ||= ":"
      d = if Fetch.gem_version_2 then
        "--document=rdoc"
      else
        "--rdoc --no-ri"
      end
      system <<-EOT
        gem install -i #{TMPDIR} #{d} --ignore-dependencies #{filename}
      EOT
    end

  end

end

